
#include "i386ArchProcessor.h"
#include "QCatagoryList.h"
#include "Debugger.h"
#include "DebuggerCoreInterface.h"
#include "DisassemblyLib.h"

//------------------------------------------------------------------------------
// Name: 
// Desc: 
//------------------------------------------------------------------------------
i386ArchProcessor::i386ArchProcessor() {

}

//------------------------------------------------------------------------------
// Name: 
// Desc: 
//------------------------------------------------------------------------------
void i386ArchProcessor::setupRegisterView(QCatagoryList *catagoryList) {
	// setup the register view
	QTreeWidgetItem *gpr = catagoryList->addCatagory("General Purpose");
	m_RegisterViewItems.push_back(catagoryList->addItem(gpr, "")); // eax
	m_RegisterViewItems.push_back(catagoryList->addItem(gpr, "")); // ebx
	m_RegisterViewItems.push_back(catagoryList->addItem(gpr, "")); // ecx
	m_RegisterViewItems.push_back(catagoryList->addItem(gpr, "")); // edx
	m_RegisterViewItems.push_back(catagoryList->addItem(gpr, "")); // ebp
	m_RegisterViewItems.push_back(catagoryList->addItem(gpr, "")); // esp
	m_RegisterViewItems.push_back(catagoryList->addItem(gpr, "")); // esi
	m_RegisterViewItems.push_back(catagoryList->addItem(gpr, "")); // edi
	m_RegisterViewItems.push_back(catagoryList->addItem(gpr, "")); // eip
	m_RegisterViewItems.push_back(catagoryList->addItem(gpr, "")); // eflags
	
	QTreeWidgetItem *segs = catagoryList->addCatagory("Segments");
	m_RegisterViewItems.push_back(catagoryList->addItem(segs, "")); // cs
	m_RegisterViewItems.push_back(catagoryList->addItem(segs, "")); // ds
	m_RegisterViewItems.push_back(catagoryList->addItem(segs, "")); // es
	m_RegisterViewItems.push_back(catagoryList->addItem(segs, "")); // fs
	m_RegisterViewItems.push_back(catagoryList->addItem(segs, "")); // gs
	m_RegisterViewItems.push_back(catagoryList->addItem(segs, "")); // ss
}

//------------------------------------------------------------------------------
// Name: 
// Desc: 
//------------------------------------------------------------------------------
QTreeWidgetItem *i386ArchProcessor::getRegisterItem(unsigned int index) {
	QTreeWidgetItem *ret = 0;
	if(index < static_cast<unsigned int>(m_RegisterViewItems.size())) {
		ret = m_RegisterViewItems[index];
	}
	return ret;
}

//------------------------------------------------------------------------------
// Name: 
// Desc: 
//------------------------------------------------------------------------------
void i386ArchProcessor::doUpdateRegister(QTreeWidgetItem *item, const QString &name, BaseTypes::reg_t value) {
	QString regString;
	// TODO: uses elided text for ascii string?
	if(Debugger::memoryRegions != 0 && Debugger::getASCIIStringAtAddress(value, regString, 4, 128)) {
		item->setText(0, QString().sprintf("%s: %08x ASCII \"%s\"", qPrintable(name), value, qPrintable(regString)));
	} else {
		item->setText(0, QString().sprintf("%s: %08x", qPrintable(name), value));
	}
}

//------------------------------------------------------------------------------
// Name: 
// Desc: 
//------------------------------------------------------------------------------
void i386ArchProcessor::updateRegisterView(const QString &defaultRegionName, const State &state) {
	if(Debugger::memoryRegions != 0) {
		doUpdateRegister(getRegisterItem(0), "EAX", state.eax);
		doUpdateRegister(getRegisterItem(1), "EBX", state.ebx);
		doUpdateRegister(getRegisterItem(2), "ECX", state.ecx);
		doUpdateRegister(getRegisterItem(3), "EDX", state.edx);
		doUpdateRegister(getRegisterItem(4), "EBP", state.ebp);
		doUpdateRegister(getRegisterItem(5), "ESP", state.esp);
		doUpdateRegister(getRegisterItem(6), "ESI", state.esi);
		doUpdateRegister(getRegisterItem(7), "EDI", state.edi);

		const QString symname = Debugger::findFunctionSymbol(state.eip, defaultRegionName);
		getRegisterItem(8)->setText(0, QString().sprintf("EIP: %08x <%s>", state.eip, qPrintable(symname)));
		getRegisterItem(9)->setText(0, QString().sprintf("EFLAGS: %08x", state.eflags));

		getRegisterItem(10)->setText(0, QString().sprintf("CS: %04x", state.cs & 0xffff));
		getRegisterItem(11)->setText(0, QString().sprintf("DS: %04x", state.ds & 0xffff));
		getRegisterItem(12)->setText(0, QString().sprintf("ES: %04x", state.es & 0xffff));
		getRegisterItem(13)->setText(0, QString().sprintf("FS: %04x", state.fs & 0xffff));
		getRegisterItem(14)->setText(0, QString().sprintf("GS: %04x", state.gs & 0xffff));
		getRegisterItem(15)->setText(0, QString().sprintf("SS: %04x", state.ss & 0xffff));
	} else {
		getRegisterItem(0)->setText(0, "EAX: 00000000");
		getRegisterItem(1)->setText(0, "EBX: 00000000");
		getRegisterItem(2)->setText(0, "ECX: 00000000");
		getRegisterItem(3)->setText(0, "EDX: 00000000");
		getRegisterItem(4)->setText(0, "EBP: 00000000");
		getRegisterItem(5)->setText(0, "ESP: 00000000");
		getRegisterItem(6)->setText(0, "ESI: 00000000");
		getRegisterItem(7)->setText(0, "EDI: 00000000");
		getRegisterItem(8)->setText(0, "EIP: 00000000");
		getRegisterItem(9)->setText(0, "EFLAGS: 00000000");
		getRegisterItem(10)->setText(0, "CS: 0000");
		getRegisterItem(11)->setText(0, "DS: 0000");
		getRegisterItem(12)->setText(0, "ES: 0000");
		getRegisterItem(13)->setText(0, "FS: 0000");
		getRegisterItem(14)->setText(0, "GS: 0000");
		getRegisterItem(15)->setText(0, "SS: 0000");
	}
}

//------------------------------------------------------------------------------
// Name: 
// Desc: 
//------------------------------------------------------------------------------
QStringList i386ArchProcessor::updateInstructionInfo(BaseTypes::address_t address, const State &state) {

	QStringList ret;
	
	if(Debugger::debuggerBase != 0) {
		uint8_t buffer[MAX_INSN_SIZE];
		Instruction insn;
		
		// TODO: make sure we could read here!
		Debugger::debuggerBase->readBytes(address, buffer, sizeof(buffer));
		const int size = insn.disassemble(buffer, sizeof(buffer), address, 0);
		if(size != 0) {
		
			// fetch the 3 possible operands
			const Operand *const operands[3] = {
				insn.operand_1st(),
				insn.operand_2nd(),
				insn.operand_3rd()
			};

			// figure out the instruction type and display some information about it
			switch(insn.type()) {
			case insn_return:
				do {
					BaseTypes::address_t returnAddress;
					Debugger::debuggerBase->readBytes(state.esp, &returnAddress, sizeof(returnAddress));
					
					const QString symname = Debugger::findFunctionSymbol(returnAddress, "");
					if(!symname.isEmpty()) {
						ret.push_back(QString().sprintf("Return to %08x <%s>", returnAddress, qPrintable(symname)));
					} else {
						ret.push_back(QString().sprintf("Return to %08x", returnAddress));
					}
				} while(0);
				break;
			case insn_jcc:
				do {
					bool taken = true;
					const bool cf = (state.eflags & 0x0001) != 0;
					const bool zf = (state.eflags & 0x0040) != 0;
					const bool of = (state.eflags & 0x0800) != 0;
					const bool df = (state.eflags & 0x0400) != 0;
					const bool sf = (state.eflags & 0x0080) != 0;
					const bool pf = (state.eflags & 0x0004) != 0;

					if((insn.flagsTested() & insn_carry_set)				&& !cf) taken = false;
					if((insn.flagsTested() & insn_zero_set)					&& !zf) taken = false;
					if((insn.flagsTested() & insn_oflow_set)				&& !of) taken = false;
					if((insn.flagsTested() & insn_dir_set)					&& !df) taken = false;
					if((insn.flagsTested() & insn_sign_set)					&& !sf) taken = false;
					if((insn.flagsTested() & insn_parity_set)				&& !pf) taken = false;
					if((insn.flagsTested() & insn_carry_clear)				&& cf) taken = false;
					if((insn.flagsTested() & insn_zero_clear)				&& zf) taken = false;
					if((insn.flagsTested() & insn_oflow_clear)				&& of) taken = false;
					if((insn.flagsTested() & insn_dir_clear)				&& df) taken = false;
					if((insn.flagsTested() & insn_sign_clear)				&& sf) taken = false;
					if((insn.flagsTested() & insn_parity_clear)				&& pf) taken = false;

					if((insn.flagsTested() & insn_zero_set_or_sign_ne_oflow)	&& (!zf && !(sf != of)))	taken = false;
					if((insn.flagsTested() & insn_carry_or_zero_set)			&& !(cf && zf))	taken = false;
					if((insn.flagsTested() & insn_sign_eq_oflow)				&& !(sf == of))	taken = false;
					if((insn.flagsTested() & insn_sign_ne_oflow)				&& (sf == of))	taken = false;

					if(taken) {
						ret.push_back("jump taken");
					} else {
						ret.push_back("jump NOT taken");
					}
				} while(0);
				// FALL THROUGH!
			case insn_jmp:
			case insn_call:
			case insn_callcc:
				if(operands[0] != 0) {
					switch(operands[0]->type()) {
					case op_relative_far:
					case op_relative_near:
					case op_register:
						do {
							const BaseTypes::address_t effective_address = operands[0]->effectiveAddress(state);

							char tmp[100];
							operands[0]->format(tmp, sizeof(tmp), Debugger::config()->syntax);

							const QString symname = Debugger::findFunctionSymbol(effective_address, "");
							if(!symname.isEmpty()) {
								ret.push_back(QString().sprintf("%s = %08x <%s>", tmp, effective_address, qPrintable(symname)));
							} else {
								ret.push_back(QString().sprintf("%s = %08x", tmp, effective_address));
							}
						} while(0);
						break;

					case op_expression:
					default:
						do {
						
							const BaseTypes::address_t effective_address = operands[0]->effectiveAddress(state);
							uint32_t target;

							char tmp[100];
							operands[0]->format(tmp, sizeof(tmp), Debugger::config()->syntax);

							const bool ok = Debugger::debuggerBase->readBytes(effective_address, &target, sizeof(target));
							
							if(ok) {
								const QString symname = Debugger::findFunctionSymbol(target, "");
								if(!symname.isEmpty()) {
									ret.push_back(QString().sprintf("%s = [%08x] = %08x <%s>", tmp, effective_address, target, qPrintable(symname)));
								} else {
									ret.push_back(QString().sprintf("%s = [%08x] = %08x", tmp, effective_address, target));
								}
							} else {
								ret.push_back(QString().sprintf("%s = [%08x] = ?", tmp, effective_address));
							}
						} while(0);
						break;
					}
				}
				break;
			default:
				for(int j = 0; j < 3; ++j) {

					if(operands[j] != 0) {
						switch(operands[j]->type()) {
						case op_relative_far:
						case op_relative_near:
						case op_register:
							do {
								const BaseTypes::address_t effective_address = operands[j]->effectiveAddress(state);

								char tmp[100];
								operands[j]->format(tmp, sizeof(tmp), Debugger::config()->syntax);

								ret.push_back(QString().sprintf("%s = %08x", tmp, effective_address));
							} while(0);
							break;
						case op_expression:
							do {
								const BaseTypes::address_t effective_address = operands[j]->effectiveAddress(state);
								uint32_t target;

								const bool ok = Debugger::debuggerBase->readBytes(effective_address, &target, sizeof(target));

								char tmp[100];
								operands[j]->format(tmp, sizeof(tmp), Debugger::config()->syntax);

								if(ok) {
									ret.push_back(QString().sprintf("%s = [%08x] = %08x", tmp, effective_address, target));
								} else {
									ret.push_back(QString().sprintf("%s = [%08x] = ?", tmp, effective_address));
								}
							} while(0);
							break;
						default:
							break;
						}
					}
				}
				break;

			}
		}
	}
	return ret;
}
